@@ -7,19 +7,28 @@ XBase::Index - base class for the index files for dbf
 
 package XBase::Index;
 use strict;
-use vars qw( @ISA $DEBUG $VERSION $VERBOSE );
+use vars qw( @ISA $DEBUG $VERSION $VERBOSE $BIGEND );
 use XBase::Base;
 @ISA = qw( XBase::Base );
 
-$VERSION = '0.162';
+$VERSION = '0.200';
 
 $DEBUG = 0;
 
 $VERBOSE = 0 unless defined $VERBOSE;
 
+# We will setup global variable to denote the byte order (endian)
+my $packed = pack('d', 1);
+if ($packed eq "\077\360\000\000\000\000\000\000") {
+	$BIGEND = 1;
+} elsif ($packed eq "\000\000\000\000\000\000\360\077") {
+	$BIGEND = 0;
+} else {
+	die "XBase::Index: your architecture is not supported.\n";
+}
+
 # Open appropriate index file and create object according to suffix
-sub new
-	{
+sub new {
 	my ($class, $file) = (shift, shift);
 	my @opts = @_;
 print "XBase::Index::new($class, $file, @_)\n" if $XBase::Index::VERBOSE;
@@ -37,15 +46,13 @@ print "XBase::Index::new($class, $file, @_)\n" if $XBase::Index::VERBOSE;
 
 	__PACKAGE__->Error("Error loading index: unknown extension\n") if $@;
 	return;
-	}
+}
 
 # For XBase::*x object, a record is one page, object XBase::*x::Page here
-sub get_record
-	{
+sub get_record {
 	my ($self, $num) = @_;
 	return $self->{'pages_cache'}{$num}
-		if defined $self->{'pages_cache'}
-			and defined $self->{'pages_cache'}{$num};
+			if defined $self->{'pages_cache'}{$num};
 
 	my $newpage = (ref $self) . '::Page::new';
 	my $page = $self->$newpage($num);
@@ -56,17 +63,16 @@ sub get_record
 		local $^W = 0;
 		print "Page $page->{'num'}:\tkeys: @{[ map { s/\s+$//; $_; } @{$page->{'keys'}}]}\n\tvalues: @{$page->{'values'}}\n" if $DEBUG;
 		print "\tlefts: @{$page->{'lefts'}}\n" if defined $page->{'lefts'} and $DEBUG;
-		}
-	$page;
 	}
+	$page;
+}
 
 # Get next (value, record number in dbf) pair
 # The important values of the index object are 'level' holding the
-# current level of the "cursor", 'pages' holding an array of pages for
-# each level (currently open) and 'rows' with an array of current row
+# current level of the "cursor", 'pages' holding an array of pages
+# currently open for each level and 'rows' with an array of current row
 # in each level
-sub fetch
-	{
+sub fetch {
 	my $self = shift;
 	my ($level, $page, $row, $key, $val, $left);
 	
@@ -75,52 +81,50 @@ sub fetch
 	while (not defined $val)
 		{
 		$level = $self->{'level'};
-		if (not defined $level)
-			{	# if we do not have level, let's start from zero
+
+		# if we do not have level, let's start from zero
+		if (not defined $level) {
 			$level = $self->{'level'} = 0;
 			$page = $self->get_record($self->{'start_page'});
-			if (not defined $page)
-				{
+			if (not defined $page) {
 				$self->Error("Index corrupt: $self: no root page $self->{'start_page'}\n");
 				return;
-				}
+			}
 			# and initialize 'pages' and 'rows'
 			$self->{'pages'} = [ $page ];
 			$self->{'rows'} = [];
-			}
+		}
 
 		# get current page for this level
 		$page = $self->{'pages'}[$level];
-		if (not defined $page)
-			{
+		if (not defined $page) {
 			$self->Error("Index corrupt: $self: page for level $level lost in normal course\n");
 			return;
-			}
+		}
 
 		# get current row for current level and increase it
 		# (or setup to zero)
 		my $row = $self->{'rows'}[$level];
-		if (not defined $row)
-			{ $row = $self->{'rows'}[$level] = 0; }
-		else
-			{ $self->{'rows'}[$level] = ++$row; }
+		if (not defined $row) {
+			$row = $self->{'rows'}[$level] = 0;
+		} else {
+			$self->{'rows'}[$level] = ++$row;
+		}
 
 		# get the (key, value, pointer) from the page
 		($key, $val, $left) = $page->get_key_val_left($row);
 
 		# there is another page to walk
-		if (defined $left)
-			{
+		if (defined $left) {
 			# go deeper
 			$level++;
 			my $oldpage = $page;
 			# load the next page
 			$page = $self->get_record($left);
-			if (not defined $page)
-				{
+			if (not defined $page) {
 				$self->Error("Index corrupt: $self: no page $left, ref'd from $oldpage, row $row, level $level\n");
 				return;
-				}
+			}
 			# and put it into the structure
 			$self->{'pages'}[$level] = $page;
 			$self->{'rows'}[$level] = undef;
@@ -130,17 +134,15 @@ sub fetch
 			# skip it when going down
 			$val = undef;
 			next;
-			}
+		}
 		# if we're lucky and got the value, return it	
-		if (defined $val)
-			{
+		if (defined $val) {
 			return ($key, $val);
-			}
+		}
 		# we neither got link to lower page, nor the value
 		# so it means we are backtracking the structure one
 		# (or more) levels back
-		else
-			{
+		else {
 			$self->{'level'} = --$level;	# go up the levels
 			return if $level < 0;		# do not fall over 
 			$page = $self->{'pages'}[$level];
@@ -148,25 +150,26 @@ sub fetch
 				{
 				$self->Error("Index corrupt: $self: page for level $level lost when backtracking\n");
 				return;
-				}
+			}
 			### next unless defined $page;
 			$row = $self->{'rows'}[$level];
 			my ($backkey, $backval, $backleft) = $page->get_key_val_left($row);
 			# this is a hook for ntx files where we do not
 			# want to miss a values that are stored inside
 			# the structure, not only in leaves.
-			if (not defined $page->{'last_key_is_just_overflow'} and defined $backleft and defined $backval)
-				{ return ($backkey, $backval); }
+			if (not defined $page->{'last_key_is_just_overflow'} and defined $backleft and defined $backval) {
+				return ($backkey, $backval);
 			}
 		}
-	return;	
 	}
+	return;	
+}
 
 # Get list of tags in the indexfile (an indexfile may not have any)
 sub tags {
 	my $self = shift;
 	@{$self->{'tags'}} if defined $self->{'tags'};
-	}
+}
 
 # Method allowing to refetch the active values (key, val) without
 # rolling forward
@@ -177,24 +180,22 @@ sub fetch_current {
 	my $row = $self->{'rows'}[$level];
 	my ($key, $val, $left) = $page->get_key_val_left($row);
 	return ($key, $val);
-	}
+}
 
 # Rewind the index to start
 # the easiest way to do this is to cancel the 'level' -- this way we
 # do not know where we are and we have to start anew
-sub prepare_select
-	{
+sub prepare_select {
 	my $self = shift;
 	delete $self->{'level'};
 	delete $self->{'pages'};
 	delete $self->{'rows'};
 	1;
-	}
+}
 
 # Position index to a value (or behind it, if nothing found), so that
 # next fetch fetches the correct value
-sub prepare_select_eq
-	{
+sub prepare_select_eq {
 	my ($self, $eq, $recno) = @_;
 	$self->prepare_select();		# start from scratch
 
@@ -207,21 +208,24 @@ sub prepare_select_eq
 	# we'll need to know if we want numeric or string compares
 	my $numdate = ($self->{'key_type'} ? 1 : 0);
 
-	while (1)
-		{
+	while (1) {
 		my $page = $self->get_record($left);	# get page
-		if (not defined $page)
-			{
+		if (not defined $page) {
 			$self->Error("Index corrupt: $self: no page $left for level $level\n");
 			return;
-			}
+		}
 		my $row = 0;
 		my ($key, $val);
-		while (($key, $val, my $newleft) = $page->get_key_val_left($row))
-			{
+		while (($key, $val, my $newleft) = $page->get_key_val_left($row)) {
 ### { local $^W = 0; print "Got: $key, $val, $newleft ($numdate)\n"; }
 
 			$left = $newleft;
+# Joe Campbell says:
+# Compound char keys have two parts preceded by white space
+# get rid of the white space so that I can do a matching....
+# and suggests
+#			$key =~ s/^\s*//g;
+
 
 			# finish if we are at the end of the page or
 			# behind the correct value
@@ -230,22 +234,22 @@ sub prepare_select_eq
 			if ($numdate == 1 ? $key >= $eq : $key ge $eq)
 				{ last; }
 			$row++;
-			}
+		}
 		
 		# we know where we are positioned on the page now
 		$self->{'pages'}[$level] = $page;
 		$self->{'rows'}[$level] = $row;
 
-		if (not defined $left)		# if there is no lower level
-			{
+		# if there is no lower level
+		if (not defined $left) {
 			$self->{'rows'}[$level] = ( $row ? $row - 1: undef);
 			$self->{'level'} = $level;
 			last;
-			}
+		}
 		$page->{'parent'} = $parent->{'num'} if defined $parent;
 		$parent = $page;
 		$level++;
-		}
+	}
 	if (defined $recno) {		# exact match requested
 		# get current values
 		my ($key, $val) = $self->fetch_current;
@@ -257,14 +261,13 @@ sub prepare_select_eq
 
 			# move forward
 			($key, $val) = $self->fetch;
-			}
 		}
-	1;
 	}
+	1;
+}
 
 # Get (key, dbf record number, lower page index) from the index page
-sub get_key_val_left
-	{
+sub get_key_val_left {
 	my ($self, $num) = @_;
 	{
 		local $^W = 0;
@@ -277,10 +280,11 @@ sub get_key_val_left
 				if $num <= $#{$self->{'keys'}};
 	}
 	return;
-	}
+}
 
-sub num_keys
-	{ $#{shift->{'keys'}}; }
+sub num_keys {
+	$#{shift->{'keys'}};
+}
 
 sub delete {
 	my ($self, $key, $value) = @_;
@@ -292,10 +296,10 @@ sub delete {
 			and $foundkey eq $key and $foundvalue == $value) {
 		$self->delete_current;
 		return 1;
-		}
+	}
 	print "$key/$value is not in the index (wanted to delete)\n" if $XBase::Index::VERBOSE;
 	undef;
-	}
+}
 sub insert {
 	my ($self, $key, $value) = @_;
 	print "XBase::Index::insert($key, $value) called\n" if $XBase::Index::VERBOSE;
@@ -307,10 +311,10 @@ sub insert {
 			and $foundkey eq $key and $foundvalue == $value) {
 		print STDERR "Already found, strange.\n";
 		return;
-		}
+	}
 
 	$self->insert_before_current($key, $value);
-	}
+}
 
 sub delete_current {
 	my $self = shift;
@@ -326,14 +330,14 @@ sub delete_current {
 	$self->{'rows'}[$level]--;
 	if ($self->{'rows'}[$level] < 0) {
 		$self->{'rows'}[$level] = undef;
-		}
+	}
 
 	$page->write_with_context;
 
 	delete $self->{'pages_cache'};
 
 	print STDERR "Delete_current returning\n" if $DEBUG;
-	}
+}
 
 sub insert_before_current {
 	my ($self, $key, $value) = @_;
@@ -353,7 +357,7 @@ sub insert_before_current {
 	delete $self->{'pages_cache'};
 
 	print STDERR "Insert_current returning\n" if $DEBUG;
-	}
+}
 
 # #############
 # dBase III NDX
@@ -365,8 +369,7 @@ use vars qw( @ISA $DEBUG );
 
 *DEBUG = \$XBase::Index::DEBUG;
 
-sub read_header
-	{
+sub read_header {
 	my $self = shift;
 	my %opts = @_;
 	my $header;
@@ -382,10 +385,11 @@ sub read_header
 	$self->{'header_len'} = 0;
 
 	$self;
-	}
+}
 
-sub last_record
-	{ shift->{'total_pages'}; }
+sub last_record {
+	shift->{'total_pages'};
+}
 
 package XBase::ndx::Page;
 use strict;
@@ -395,15 +399,14 @@ use vars qw( @ISA $DEBUG );
 *DEBUG = \$XBase::Index::DEBUG;
 
 # Constructor for the ndx page
-sub new
-	{
+sub new {
 	my ($indexfile, $num) = @_;
 	my $parent;
-	if ((ref $indexfile) =~ /::Page$/)
-		{			# we can be called from parent page
+	# we can be called from parent page
+	if ((ref $indexfile) =~ /::Page$/) {			
 		$parent = $indexfile;
 		$indexfile = $parent->{'indexfile'};
-		}
+	}
 	
 	my $data = $indexfile->read_record($num) or return;	# get 512 bytes
 	my $noentries = unpack 'V', $data;			# num of entries
@@ -413,22 +416,20 @@ sub new
 
 	print "page $num, noentries $noentries, keylength $keylength\n" if $DEBUG;
 	my $numdate = $indexfile->{'key_type'};		# numeric or string?
-	my $bigend = substr(pack('d', 1), 0, 2) eq '?';	# endian
 	
 	my $offset = 4;
 	my $i = 0;
 	my ($keys, $values, $lefts) = ([], [], []);		# three arrays
 
-	while ($i < $noentries)				# walk the page
-		{
+	# walk the page
+	while ($i < $noentries) {
 		# get the values for entry
 		my ($left, $recno, $key)
 			= unpack 'VVa*', substr($data, $offset, $keylength + 8);
-		if ($numdate)
-			{			# some decoding for numbers
-			$key = reverse $key if $bigend;
+		if ($numdate) {			# some decoding for numbers
+			$key = reverse $key if $XBase::Index::BIGEND;
 			$key = unpack 'd', $key;
-			}
+		}
 		print "$i: \@$offset VVa$keylength -> ($left, $recno, $key)\n" if $DEBUG > 1;
 		push @$keys, $key;
 		push @$values, ($recno ? $recno : undef);
@@ -438,12 +439,11 @@ sub new
 		if ($i == 0 and defined $left)
 			{ $noentries++; }	# fixup for nonleaf page
 				### shouldn't this be for last page only?
-		}
-	continue
-		{
+	}
+	continue {
 		$i++;
 		$offset += $keyreclength;
-		}
+	}
 
 	my $self = bless { 'keys' => $keys, 'values' => $values,
 		'num' => $num, 'keylength' => $keylength,
@@ -454,10 +454,10 @@ sub new
 			$parent->{'last_key_is_just_overflow'} and
 			$parent->{'lefts'}[$#{$parent->{'lefts'}}] == $num)) {
 		$self->{'last_key_is_just_overflow'} = 1;
-		}
+	}
 
 	$self;
-	}
+}
 
 # ###########
 # Clipper NTX
@@ -467,8 +467,7 @@ use strict;
 use vars qw( @ISA $DEBUG );
 @ISA = qw( XBase::Base XBase::Index );
 
-sub read_header
-	{
+sub read_header {
 	my $self = shift;
 	my %opts = @_;
 	my $header;
@@ -488,7 +487,7 @@ sub read_header
 	if ($self->{'signature'} != 3 and $self->{'signature'} != 6) {
 		__PACKAGE__->Error("$self: bad signature value `$self->{'signature'}' found\n");
 		return;
-		}
+	}
 	$self->{'key_string'} =~ s/[\000 ].*$//s;
 	$self->{'record_len'} = 1024;
 	$self->{'header_len'} = 0;
@@ -500,21 +499,22 @@ sub read_header
 		if (not defined $field_type) {
 			__PACKAGE__->Error("Couldn't find key string `$key_string' in dbf file, can't determine field type\n");
 			return;
-			}
 		}
+	}
 	elsif (defined $opts{'type'}) {
 		$field_type = $opts{'type'};
-		}
+	}
 	else {
 		__PACKAGE__->Error("Index type (char/numeric) unknown for $self\n");
 		return;
-		}
+	}
 	$self->{'key_type'} = ($field_type =~ /^[NDIF]$/ ? 1 : 0);
 
 	$self;
-	}
-sub last_record
-	{ -1; }
+}
+sub last_record {
+	-1;
+}
 
 
 package XBase::ntx::Page;
@@ -525,15 +525,14 @@ use vars qw( @ISA $DEBUG );
 *DEBUG = \$XBase::Index::DEBUG;
 
 # Constructor for the ntx page
-sub new
-	{
+sub new {
 	my ($indexfile, $num) = @_;
 	my $parent;
-	if ((ref $indexfile) =~ /::Page$/)
-		{			# we could be called from parent page
+	# we could be called from parent page
+	if ((ref $indexfile) =~ /::Page$/) {			
 		$parent = $indexfile;
 		$indexfile = $parent->{'indexfile'};
-		}
+	}
 	my $data = $indexfile->read_record($num) or return;	# get data
 	my $maxnumitem = $indexfile->{'max_item'} + 1;	# limit from header
 	my $keylength = $indexfile->{'key_length'};
@@ -547,20 +546,19 @@ sub new
 	print "page $num, noentries $noentries, keylength $keylength; pointers @pointers\n" if $DEBUG;
 	
 	my ($keys, $values, $lefts) = ([], [], []);
-	for (my $i = 0; $i < $noentries; $i++)		# walk the pointers
-		{
+	# walk the pointers
+	for (my $i = 0; $i < $noentries; $i++) {
 		my $offset = $pointers[$i];
 		my ($left, $recno, $key)
 			= unpack 'VVa*', substr($data, $offset, $keylength + 8);
 
-		if ($numdate)
-			{
+		if ($numdate) {
 			### if looks like with ntx the numbers are
 			### stored as ASCII strings or something
 			### To Be Done
 			if ($key =~ tr!,+*)('&%$#"!0123456789!) { $key = '-' . $key; }
 			$key += 0;
-			}
+		}
 
 		print "$i: \@$offset VVa$keylength -> ($left, $recno, $key)\n" if $DEBUG > 1;
 		push @$keys, $key;
@@ -572,13 +570,13 @@ sub new
 		if ($i == 0 and defined $left)
 			{ $noentries++; }
 				### shouldn't this be for last page only?
-		}
+	}
 
 	my $self = bless { 'num' => $num, 'indexfile' => $indexfile,
 		'keys' => $keys, 'values' => $values, 'lefts' => $lefts, },
 								__PACKAGE__;
 	$self;
-	}
+}
 
 # ###########
 # FoxBase IDX
@@ -590,8 +588,7 @@ use vars qw( @ISA $DEBUG );
 
 *DEBUG = \$XBase::Index::DEBUG;
 
-sub read_header
-	{
+sub read_header {
 	my $self = shift;
 	my %opts = @_;
 	my $header;
@@ -611,11 +608,119 @@ sub read_header
 	$self->{'start_free_list'} /= $self->{'record_len'};
 	$self->{'header_len'} = 0;
 
+	if ($opts{'type'} eq 'N') {
+		$self->{'key_type'} = 1;
+		}
+
 	$self;
+}
+
+sub last_record {
+	shift->{'total_pages'};
+}
+
+sub create {
+	my ($class, $table, $filename, $column) = @_;
+	my $type = $table->field_type($column);
+	if (not defined $type) {
+		die "XBase::idx: could determine index type for `$column'\n";
+	}
+	my $numdate = 0;
+	$numdate = 1 if $type eq 'N' or $type eq 'D';
+
+	my $self = bless {}, $class;
+	$self->create_file($filename) or die "Error creating `$filename'\n";
+	$self->write_to(0, "\000" x 512);
+	my $key_length = $table->field_length($column);
+	$key_length = 8 if $numdate;
+
+	my $count = int((512 - 12) / ($key_length + 4));
+### warn "Key length $key_length, per page $count.\n";
+
+	my $encode_function;
+	if ($numdate) {
+		$encode_function = sub {
+			my $key = pack 'd', shift;
+			$key = reverse $key unless $XBase::Index::BIGEND;
+			if ((substr($key, 0, 1) & "\200") eq "\200") {
+				$key ^= "\377\377\377\377\377\377\377\377";
+			} else {
+				$key ^= "\200";
+			}
+			return $key;
+		};
+	} else {
+		$encode_function = sub {
+			return sprintf "%-${key_length}s", shift;
+		};
 	}
 
-sub last_record
-	{ shift->{'total_pages'}; }
+	my @data;
+	my $last_record = $table->last_record;
+	for (my $i = 0; $i <= $last_record; $i++) {
+		my ($deleted, $data) = $table->get_record($i, $column);
+		push @data, [ $encode_function->($data), $i + 1 ];
+	}
+	@data = sort { $a->[0] cmp $b->[0] } @data;
+
+	$self->{'header_len'} = 0;	# it is 512 really, but we
+					# count from 1, not from 0
+	$self->{'record_len'} = 512;
+
+	my $pageno = 1;
+	my $level = 1;
+	my @newdata;
+	while ($level == 1 or @data > 1) {
+		last if $pageno > 5;
+		my $attributes = 0;
+		$attributes = 2 if $level == 1;
+		if (scalar(@data) < $count) {
+			# we have less than one page, so it's root.
+			$attributes++;	
+		}
+
+		my $left_page = 0xFFFFFFFF;
+		my $current_count = 0;
+		my $out = '';
+		@newdata = ();
+		for (my $i = 0; $i < @data; $i++) {
+			my $key = $data[$i][0];
+### print STDERR "Page $pageno: $i: @{$data[$i]}\n";
+			$out .= pack "a$key_length N", $key, $data[$i][1];
+			$current_count++;
+
+			if ($current_count == $count or $i == $#data) {
+### print STDERR "Dumping $pageno.\n";
+				# time to close this page and move on
+				my $right_page = 0xFFFFFFFF;
+				if ($i < $#data) {
+					$right_page = $pageno + 1;
+				}
+				$self->write_record($pageno,
+					pack 'a512',
+						pack('vvVV', $attributes, $current_count,
+						$left_page, $right_page)
+					. $out);
+				push @newdata, [$data[$i][0], $pageno * 512];
+				$left_page = $pageno;
+				$current_count = 0;
+				$pageno++;
+				$out = '';
+			}
+		}
+
+		@data = @newdata;
+		$level++;
+	}
+
+	my $header = pack 'VVVv CC a220 a276',
+		($pageno - 1) * 512, 0xFFFFFFFF, $pageno * 512,
+		$key_length, 0, 0, $column, '';
+	$self->write_to(0, $header);
+	$self->close;
+
+	return new XBase::Index($filename, 'type' => $type);
+}
 
 package XBase::idx::Page;
 use strict;
@@ -624,16 +729,17 @@ use vars qw( @ISA $DEBUG );
 
 *DEBUG = \$XBase::Index::DEBUG;
 
+### $DEBUG = 1;
 # Constructor for the idx page
-sub new
-	{
+sub new {
+	local $^W = 0;
 	my ($indexfile, $num) = @_;
 	my $parent;
-	if ((ref $indexfile) =~ /::Page$/)
-		{			# we can be called from parent page
+	# we can be called from parent page
+	if ((ref $indexfile) =~ /::Page$/) {			
 		$parent = $indexfile;
 		$indexfile = $parent->{'indexfile'};
-		}
+	}
 	my $data = $indexfile->read_record($num) or return;	# get 512 bytes
 	my ($attributes, $noentries, $left_brother, $right_brother)
 		= unpack 'vvVV', $data;		# parse header of the page
@@ -642,27 +748,33 @@ sub new
 
 	print "page $num, noentries $noentries, keylength $keylength\n" if $DEBUG;
 	my $numdate = $indexfile->{'key_type'};		# numeric or string?
-	my $bigend = substr(pack('d', 1), 0, 2) eq '?';	# endian
 	
 	my $offset = 12;
 	my $i = 0;
 	my ($keys, $values, $lefts) = ([], [], []);		# three arrays
 
-	while ($i < $noentries)				# walk the page
-		{
+	# walk the page
+	while ($i < $noentries) {
 		# get the values for entry
 		my ($key, $recno) = unpack "\@$offset a$keylength N", $data;
 		my $left;
 		unless ($attributes & 2) {
-			$left = $recno;
+			$left = $recno / 512;
 			$recno = undef;
+		}
+		print "$i: \@$offset a$keylength N -> ($left, $recno, $key)\n" if $DEBUG > 1;
+		### use Data::Dumper; print Dumper $indexfile;
+		# some decoding for numbers
+		if ($numdate) {			
+			if ((substr($key, 0, 1) & "\200") ne "\200") {
+				$key ^= "\377\377\377\377\377\377\377\377";
+			} else {
+				$key ^= "\200";
 			}
-		if ($numdate)
-			{			# some decoding for numbers
-			$key = reverse $key if $bigend;
+			if (not $XBase::Index::BIGEND) { $key = reverse $key; }
 			$key = unpack 'd', $key;
-			}
-		print "$i: \@$offset VVa$keylength -> ($left, $recno, $key)\n" if $DEBUG > 1;
+		}
+		print "$i: \@$offset a$keylength N -> ($left, $recno, $key)\n" if $DEBUG > 1;
 		push @$keys, $key;
 		push @$values, ($recno ? $recno : undef);
 		$left = ($left ? $left : undef);
@@ -671,12 +783,11 @@ sub new
 		if ($i == 0 and defined $left)
 			{ $noentries++; }	# fixup for nonleaf page
 				### shouldn't this be for last page only?
-		}
-	continue
-		{
+	}
+	continue {
 		$i++;
 		$offset += $keyreclength;
-		}
+	}
 
 	my $self = bless { 'keys' => $keys, 'values' => $values,
 		'num' => $num, 'keylength' => $keylength,
@@ -685,7 +796,7 @@ sub new
 		'left_brother' => $left_brother,
 		'right_brother' => $right_brother }, __PACKAGE__;
 	$self;
-	}
+}
 
 # ############
 # dBase IV MDX
@@ -695,8 +806,7 @@ use strict;
 use vars qw( @ISA $DEBUG );
 @ISA = qw( XBase::Base XBase::Index );
 
-sub read_header
-	{
+sub read_header {
 	my $self = shift;
 	my %opts = @_;
 	my $expr_name = $opts{'tag'};
@@ -714,8 +824,7 @@ sub read_header
 	$self->{'record_len'} = 512;
 	$self->{'header_len'} = 0;
 
-	for my $i (1 .. $self->{'tags_used'})
-		{
+	for my $i (1 .. $self->{'tags_used'}) {
 		my $len = $self->{'tag_length'};
 		
 		$self->seek_to(544 + ($i - 1) * $len) or do
@@ -741,21 +850,21 @@ sub read_header
 			key_type_1 res key_length max_no_keys_per_page
 			second_key_type key_record_length res unique) }
 				 = unpack 'VVca1vvvvva3c', $header;
-		}
+	}
 
-## use Data::Dumper;
-## print Dumper $self;
+### use Data::Dumper; print Dumper $self;
 
-	if (defined $expr_name and defined $self->{'tags'}{$expr_name})
-		{
+	if (defined $expr_name and defined $self->{'tags'}{$expr_name}) {
 		$self->{'active'} = $self->{'tags'}{$expr_name};
 		$self->{'start_page'} = $self->{'active'}{'start_page'};
-		}
+	}
 
 	$self;
-	}
-sub last_record
-	{ -1; }
+}
+
+sub last_record {
+	-1;
+}
 
 package XBase::mdx::Page;
 use strict;
@@ -765,16 +874,15 @@ use vars qw( @ISA $DEBUG );
 *DEBUG = \$XBase::Index::DEBUG;
 
 # Constructor for the mdx page
-sub new
-	{
+sub new {
 	my ($indexfile, $num) = @_;
 
 	my $parent;
-	if ((ref $indexfile) =~ /::Page$/)		### parent page
-		{
+	### parent page
+	if ((ref $indexfile) =~ /::Page$/) {
 		$parent = $indexfile;
 		$indexfile = $parent->{'indexfile'};
-		}
+	}
 	$indexfile->seek_to_record($num) or return;
 	my $data;
 	$indexfile->{'fh'}->read($data, 1024) == 1024 or return;
@@ -786,14 +894,13 @@ sub new
 	my ($noentries, $noleaf) = unpack 'VV', $data;
 
 	print "page $num, noentries $noentries, keylength $keylength; noleaf: $noleaf\n" if $DEBUG;
-	if ($noleaf == 54 or $noleaf == 20 or $noleaf == 32
-						or $noleaf == 80)
-		{ $noentries++; }
+	if ($noleaf == 54 or $noleaf == 20 or $noleaf == 32 or $noleaf == 80) {
+		$noentries++;
+	}
 
 	my ($keys, $values, $lefts) = ([], [], []);
 
-	for (my $i = 0; $i < $noentries; $i++)
-		{
+	for (my $i = 0; $i < $noentries; $i++) {
 		my ($left, $key)
 			= unpack "\@${offset}Va${keylength}", $data;
 
@@ -805,13 +912,13 @@ sub new
 		else
 			{ push @$values, $left; }
 		$offset += $keyreclength;
-		}
+	}
 
 	my $self = bless { 'num' => $num, 'indexfile' => $indexfile,
 		'keys' => $keys, 'values' => $values, 'lefts' => $lefts, },
 								__PACKAGE__;
 	$self;
-	}
+}
 
 # ###########
 # FoxBase CDX
@@ -835,15 +942,14 @@ sub prepare_write_header {
 		key_string
 		) };
 	$data;
-	}
+}
 sub write_header {
 	my $self = shift;
 	my $data = $self->prepare_write_header;
 	$self->{'fh'}->seek($self->{'adjusted_offset'} || 0, 0);
 	$self->{'fh'}->print($data);
-	}
-sub read_header
-	{
+}
+sub read_header {
 	my ($self, %opts) = @_;
 	$self->{'dbf'} = $opts{'dbf'} if not exists $self->{'dbf'};
 
@@ -877,14 +983,14 @@ sub read_header
 ## 	print STDERR "I won't be able to write the header back\n",
 ## 	unpack("H*", $out), "\n ++\n",
 ## 	unpack("H*", $header), "\n";
-## 	}
+## }
 
 	if (not defined $self->{'tag'}) {	# top level
 		$self->prepare_select;
 		while (my ($tag) = $self->fetch) {
 			push @{$self->{'tags'}}, $tag;
-			}
 		}
+	}
 ### use Data::Dumper; print Dumper \%opts;
 
 	if (defined $opts{'tag'}) {
@@ -905,34 +1011,35 @@ sub read_header
 		my $field_type;
 		if (defined $opts{'type'}) {
 			$field_type = $opts{'type'};
-			}
+		}
 		elsif (defined $subidx->{'dbf'}) {
 			$field_type = $subidx->{'dbf'}->field_type($key_string);
 			if (not defined $field_type) {
 				__PACKAGE__->Error("Couldn't find key string `$key_string' in dbf file, can't determine field type\n");
 				return;
-				}
 			}
+		}
 		else {
 			__PACKAGE__->Error("Index type (char/numeric) unknown for $subidx\n");
 			return;
-			}
+		}
 		$subidx->{'key_type'} = ($field_type =~ /^[NDIF]$/ ? 1 : 0);
 		if ($field_type eq 'D') {
 			$subidx->{'key_type'} = 2;
 			require Time::JulianDay;
-			}
+		}
 
 		for (keys %$self) { delete $self->{$_} }
 		for (keys %$subidx) { $self->{$_} = $subidx->{$_} }
 		$self = $subidx;
 ### use Data::Dumper; print Dumper $self;
-		}
-	$self;
 	}
+	$self;
+}
 
-sub last_record
-	{ shift->{'total_pages'}; }
+sub last_record {
+	shift->{'total_pages'};
+}
 
 package XBase::cdx::Page;
 use strict;
@@ -942,8 +1049,7 @@ use vars qw( @ISA $DEBUG );
 *DEBUG = \$XBase::Index::DEBUG;
 
 # Constructor for the cdx page
-sub new
-	{
+sub new {
 	my ($indexfile, $num) = @_;
 	my $data = $indexfile->read_record($num)
 		or do { print $indexfile->errstr; return; };	# get 512 bytes
@@ -957,7 +1063,6 @@ sub new
 
 	print "page $num, attr $attributes, noentries $noentries, keylength $keylength (bro $left_brother, $right_brother)\n" if $DEBUG;
 	my $numdate = $indexfile->{'key_type'};		# numeric or string?
-	my $bigend = substr(pack('d', 1), 0, 2) eq '?';	# endian
 
 	my ($keys, $values, $lefts) = ([], [], undef);
 
@@ -1005,72 +1110,66 @@ sub new
 			$prevkeyval = $key;
 
 ### print "Numdate $numdate\n";
-			if ($numdate)
-				{		# some decoding for numbers
+			if ($numdate) {		# some decoding for numbers
 ### print " *** In: ", unpack("H*", $key), "\n";
 				if (0x80 & unpack('C', $key)) {
 					substr($key, 0, 1) &= "\177";
-					}
+				}
 				else { $key = ~$key; }
 				if ($keylength == 8) {
-					$key = reverse $key unless $bigend;
+					$key = reverse $key unless $XBase::Index::BIGEND;
 					$key = unpack 'd', $key;
-					}
-				else {
+				} else {
 					$key = unpack 'N', $key;
-					}
+				}
 				if ($numdate == 2 and $key) {	# date
 					$key = sprintf "%04d%02d%02d",
 						Time::JulianDay::inverse_julian_day($key);
-					}
 				}
-			else {
+			} else {
 				substr($key, -$trail) = '' if $trail;
-				}
+			}
 
 			print "$key -> $recno\n" if $DEBUG > 4;
 			push @$keys, $key;
 			push @$values, $recno;
-			}
 		}
-	else {
+	} else {
 		for (my $i = 0; $i < $noentries; $i++) {
 			my $offset = 12 + $i * ($keylength + 8);
 			my ($key, $recno, $page)
 				= unpack "\@$offset a$keylength NN", $data;
-			if ($numdate)
-				{		# some decoding for numbers
+			# some decoding for numbers
+			if ($numdate) {		
 				if (0x80 & unpack('C', $key)) {
 				### if ("\200" & substr($key, 0, 1)) {
 ### print STDERR "Declean\n";
 ### print STDERR unpack("H*", $key), ' -> ';
 					substr($key, 0, 1) &= "\177";
 ### print STDERR unpack("H*", $key), "\n";
-					}
+				}
 				else { $key = ~$key; }
 				if ($keylength == 8) {
-					$key = reverse $key unless $bigend;
+					$key = reverse $key unless $XBase::Index::BIGEND;
 					$key = unpack 'd', $key;
-					}
-				else {
+				} else {
 					$key = unpack 'N', $key;
-					}
+				}
 				if ($numdate == 2 and $key) {	# date
 					$key = sprintf "%04d%02d%02d",
 						Time::JulianDay::inverse_julian_day($key);
-					}
 				}
-			else {
+			} else {
 				$key =~ s/\000+$//;
-				}
+			}
 			print "item: $key -> $recno via $page\n" if $DEBUG > 4;
 			push @$keys, $key;
 			push @$values, $recno;
 			$lefts = [] unless defined $lefts;
 			push @$lefts, $page / 512;
-			}
-		$opts{'last_key_is_just_overflow'} = 1;
 		}
+		$opts{'last_key_is_just_overflow'} = 1;
+	}
 
 	my $self = bless { 'keys' => $keys, 'values' => $values,
 		'num' => $num, 'keylength' => $keylength,
@@ -1085,13 +1184,12 @@ sub new
 		print "I won't be able to write this page back.\n",
 			unpack("H*", $outdata), "\n ++\n",
 			unpack("H*", $origdata), "\n";
-		}
-	else {
+	} else {
 		### print STDERR " ** Bingo: I will be able to write this page back ($num).\n";
-		}
+	}
 
 	$self;
-	}
+}
 
 # Create "new" page -- allocates memory in the file and returns
 # structure that can reasonably used as XBase::cdx::Page
@@ -1099,7 +1197,7 @@ sub create {
 	my ($class, $indexfile) = @_;
 	if (not defined $indexfile and ref $class) {
 		$indexfile = $class->{'indexfile'};
-		}
+	}
 	my $fh = $indexfile->{'fh'};
 	$fh->seek(0, 2);		# seek to the end;
 	my $position = $fh->tell;	# get the length of the file
@@ -1107,12 +1205,12 @@ sub create {
 		$fh->print("\000" x (512 - ($position % 512)));
 					# pad the file to multiply of 512
 		$position = $fh->tell;	# get the length of the file
-		}
+	}
 	$fh->print("\000" x 512);
 	return bless { 'num' => $position / 512,
 		'keylength' => $indexfile->{'key_length'},
 		'indexfile' => $indexfile }, $class;
-	}
+}
 
 sub prepare_scalar_for_write {
 	my $self = shift;
@@ -1125,7 +1223,6 @@ sub prepare_scalar_for_write {
 		$right_brother;
 	
 	my $indexfile = $self->{'indexfile'};
-	my $bigend = substr(pack('d', 1), 0, 2) eq '?';	# endian
 	my $numdate = $indexfile->{'key_type'};		# numeric or string?
 	my $record_len = $indexfile->{'record_len'};
 	my $keylength = $self->{'keylength'};
@@ -1139,7 +1236,7 @@ sub prepare_scalar_for_write {
 					$holding_recno) = 
 			@{$self}{ qw! recno_count duplicate_count trailing_count
 					holding_recno !  };
-			}
+		}
 
 ### print STDERR "Hmmm. We are setting hardcoded values for bitmasks, not good. Write to adelton.\n";
 		my ($recno_mask, $duplicate_mask, $trailing_mask)
@@ -1157,35 +1254,34 @@ sub prepare_scalar_for_write {
 			my $dupl = 0;
 
 			my $out = $key;
-			if ($numdate)
-				{		# some encoding for numbers
+			# some encoding for numbers
+			if ($numdate) {		
 				if ($keylength == 8) {
 					$out = pack 'd', $out;
-					$out = reverse $out unless $bigend;
-					}
-				else {
+					$out = reverse $out unless $XBase::Index::BIGEND;
+				} else {
 					$out = pack 'N', $out;
-					}
+				}
 
 
 				unless (0x80 & unpack('C', $out)) {
 					substr($out, 0, 1) |= "\200";
-					}
-				else { $out = ~$out; }
 				}
+				else { $out = ~$out; }
+			}
 
 			for my $i (0 .. length($out) - 1) {
 				unless (substr($out, $i, 1) eq substr($prevkey, $i, 1)) {
 					last;
-					}
+				}
 				$dupl++;
-				}	
+			}	
 
 			my $trail = $keylength - length $out;
 			while (substr($out, -1) eq "\000") {
 				$out = substr($out, 0, length($out) - 1);
 				$trail++;
-				}
+			}
 			$keys_string = substr($out, $dupl) . $keys_string;
 
 
@@ -1200,7 +1296,7 @@ sub prepare_scalar_for_write {
 
 			$prevkey = $out;
 			$row++;
-			}
+		}
 		### print $keys_string, "\n";	
 
 ### print STDERR "Hmmm. The \$numdata is really just a hack -- the shifts have to be made 64 bit clean.\n"; 
@@ -1213,37 +1309,35 @@ sub prepare_scalar_for_write {
 		$data .= $recno_data;
 		$data .= "\000" x ($record_len - length($data) - length($keys_string));
 		$data .= $keys_string;
-		}
-	else {
+	} else {
 		my $row = 0;
 		for my $key (@{$self->{'keys'}}) {
 			my $out = $key;
-			if ($numdate)
-				{		# some encoding for numbers
+			# some encoding for numbers
+			if ($numdate) {		
 				if ($keylength == 8) {
 					$out = pack 'd', $out;
-					$out = reverse $out unless $bigend;
-					}
-				else {
+					$out = reverse $out unless $XBase::Index::BIGEND;
+				} else {
 					$out = pack 'N', $out;
-					}
+				}
 
 
 				unless (0x80 & unpack('C', $out)) {
 					substr($out, 0, 1) |= "\200";
-					}
+				}
 				else { $out = ~$out; }
 ### print " *** Out2: ", unpack("H*", $out), "\n";
-				}
+			}
 			$data .= pack "a$keylength NN", $out,
 				$self->{'values'}[$row],
 				$self->{'lefts'}[$row] * 512;
 			$row++;
-			}
-		$data .= "\000" x ($record_len - length($data));
 		}
-	$data;
+		$data .= "\000" x ($record_len - length($data));
 	}
+	$data;
+}
 
 sub write_page {
 	my $self = shift;
@@ -1253,7 +1347,7 @@ sub write_page {
 	die "Data is too long in cdx::write_page for $self->{'num'}\n"
 						if length $data > 512;
 	$indexfile->write_record($self->{'num'}, $data);
-	}
+}
 
 # Saves current page, taking into account all neighbour and parent
 # pages. We can safely assume that this method is called for pages
@@ -1285,12 +1379,12 @@ sub write_with_context {
 			my $fix_brother = $indexfile->get_record($right_brother_num / 512);
 			$fix_brother->{'left_brother'} = $left_brother_num;
 			$fix_brother->write_page;
-			}
+		}
 		if ($left_brother_num != 0xFFFFFFFF) {
 			my $fix_brother = $indexfile->get_record($left_brother_num / 512);
 			$fix_brother->{'right_brother'} = $right_brother_num;
 			$fix_brother->write_page;
-			}
+		}
 
 		# now we need to release ourselves from parent as well
 		my $parent = $self->get_parent_page or die "Index corrupt: no parent for page $self ($self_num)\n";
@@ -1303,14 +1397,14 @@ sub write_with_context {
 				splice @{$parent->{'values'}}, $i, 1;
 				splice @{$parent->{'lefts'}}, $i, 1;
 				last;
-				}
 			}
+		}
 		if ($i > $maxindex) {
 			die "Index corrupt: parent doesn't point to us in write_with_context $self ($self_num)\n";
-			}
+		}
 		$parent->write_with_context;
 		return;
-		}
+	}
 
 
 	if (length $data > 512) {	# we need to split the page
@@ -1329,7 +1423,7 @@ sub write_with_context {
 		if ($half_rows == 0) { $half_rows++; }
 		if ($half_rows == $total_rows) {
 			die "Fatal trouble: page $self ($self_num) is full but I'm not able to split it\n";
-			}
+		}
 
 		# new page is right brother (will get bigger values)
 		$new_page->{'right_brother'} = $self->{'right_brother'};
@@ -1340,7 +1434,7 @@ sub write_with_context {
 			my $fix_brother = $indexfile->get_record($new_page->{'right_brother'} / 512);
 			$fix_brother->{'left_brother'} = $new_page->{'num'} * 512;
 			$fix_brother->write_page;
-			}
+		}
 
 		# we'll split keys and values
 		$new_page->{'keys'} = [ @{$self->{'keys'}}[$half_rows .. $total_rows - 1] ];
@@ -1355,10 +1449,10 @@ sub write_with_context {
 			for my $q (@{$new_page->{'lefts'}}) {
 				if (defined $q and defined $indexfile->{'pages_cache'}{$q}) {
 					$indexfile->{'pages_cache'}{$q}{'parent'} = $new_page_num;
-					}
 				}
-			splice @{$self->{'lefts'}}, $half_rows, $total_rows - $half_rows - 1;
 			}
+			splice @{$self->{'lefts'}}, $half_rows, $total_rows - $half_rows - 1;
+		}
 
 		my $parent;
 		if ($self_num == $indexfile->{'start_page'}) {
@@ -1382,8 +1476,7 @@ sub write_with_context {
 			$parent->{'values'} = [ $self->{'values'}[-1],
 						$new_page->{'values'}[-1] ];
 			$parent->{'lefts'} = [ $self_num, $new_page->{'num'} ];
-			}
-		else {	# update pointers in parent page
+		} else {	# update pointers in parent page
 			$parent = $self->get_parent_page or die "Index corrupt: no parent for page $self ($self_num)\n";
 			my $maxindex = $#{$parent->{'lefts'}};
 			my $i = 0;
@@ -1392,11 +1485,11 @@ sub write_with_context {
 			while ($i <= $maxindex) {
 				last if $parent->{'lefts'}[$i] == $self_num;
 				$i++;
-				}
+			}
 			
 			if ($i > $maxindex) {
 				die "Index corrupt: parent doesn't point to us in write_with_context $self ($self_num)\n";
-				}
+			}
 
 			# now $i is index in parent of the record pointing to us
 
@@ -1406,7 +1499,7 @@ sub write_with_context {
 				$self->{'values'}[-1], $new_page->{'values'}[-1];
 			splice @{$parent->{'lefts'}}, $i, 1,
 				$self_num, $new_page->{'num'};
-			}
+		}
 
 		$self->write_page;
 
@@ -1414,7 +1507,7 @@ sub write_with_context {
 		$new_page->write_page;
 
 		$parent->write_with_context;
-		}
+	}
 	elsif ($self_num != $indexfile->{'start_page'}) {
 		# the output data is OK, write is out
 		# but this is not root page, so we need to make sure the
@@ -1432,11 +1525,11 @@ sub write_with_context {
 		while ($i <= $maxindex) {
 			last if $parent->{'lefts'}[$i] == $self_num;
 			$i++;
-			}
+		}
 		
 		if ($i > $maxindex) {
 			die "Index corrupt: parent doesn't point to us in write_with_context $self ($self_num)\n";
-			}
+		}
 
 		# now $i is index in parent of the record pointing to us
 
@@ -1445,15 +1538,14 @@ sub write_with_context {
 			$parent->{'values'}[$i] = $self->{'values'}[-1];
 			$parent->{'keys'}[$i] = $self->{'keys'}[-1];
 			$parent->write_with_context;
-			}
+		}
 		
-		}
-	else {	# write out root page
+	} else {	# write out root page
 		$indexfile->write_record($self_num, $data);
-		}
+	}
 
 	print STDERR "XBase::cdx::Page::write_with_context finished ($self->{'num'})\n" if $DEBUG;
-	}
+}
 
 # finds parent page for the object
 sub get_parent_page_num {
@@ -1475,11 +1567,11 @@ sub get_parent_page_num {
 		if ("$self" eq "$indexfile->{'pages'}[$pageindex]") {
 			print STDERR "Parent page for $self->{'num'} is $indexfile->{'pages'}[$pageindex - 1]{'num'}.\n";
 			return $indexfile->{'pages'}[$pageindex - 1]->{'num'};
-			}
-		$pageindex--;
 		}
-	return undef;
+		$pageindex--;
 	}
+	return undef;
+}
 sub get_parent_page {
 	my $self = shift;
 	my $parent_num = $self->get_parent_page_num or return;
@@ -1502,7 +1594,7 @@ __END__
 	while (my @data = $cur->fetch()) {
 		last if $data[0] != 1097;
 		print "@data\n";
-		}
+	}
 
 This is a snippet of code to print ID and NAME fields from dbf
 data.dbf where ID equals 1097. Provided you have index on ID in
@@ -1702,7 +1794,7 @@ directory.
 
 =head1 VERSION
 
-0.162
+0.200
 
 =head1 AUTHOR
 
